/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.Date;

// Import clauses required by the object Cmp2_Ship
import jorm.Cmp2_ShipAccessor;
// Imports needed by the rdb mapper
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.util.HashMap;

public class Cmp2_ShipBinding
	extends org.objectweb.jonas_ejb.container.JEntitySwitch
	implements PBindingCtrl
 
 {
	/**
	 * The current state of this PBinding object.
	 */
	protected byte status = PBinding.LIFECYCLE_NOTBOUND;

	/**
	 * The PClassMapping linked to this PBinding object.
	 */
	protected Cmp2_ShipMapping pClassMapping = null;

	/**
	 * The PName of this PBinding object.
	 */
	protected PName pname = null;

	// Declare the Connection and the ResultSet used (java.sql)
	private Connection connection = null;
	private ResultSet resultSet = null;

	public Cmp2_ShipBinding() {
		super();
// nothing
	}

	public Cmp2_ShipBinding(PClassMapping pcm) {
		super();
		pClassMapping = (Cmp2_ShipMapping) pcm;
// nothing
	}


	// IMPLEMENTATION OF METHODS FROM THE PBinding INTERFACE

	public void init(PClassMapping pcm) throws PException {
		pClassMapping = (Cmp2_ShipMapping) pcm;
	}

	public synchronized void write(Object conn, PAccessor pa)
		throws PException {
		Cmp2_ShipAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_ShipAccessor)
			paccessor = (Cmp2_ShipAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to write without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to write with this connection type.");
        PreparedStatement pstmt = null;
		try {
			Blob blob;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Remove");
            pstmt = connection.prepareStatement("DELETE FROM CMP2_SHIP_ WHERE id_ = ? ");
 
            java.lang.Integer id_ = pname.encodeOint();
            if (id_==null) {
                pstmt.setNull(1, 4);
            } else {
                pstmt.setObject(1, id_);
            }
            pstmt.executeUpdate();
        }
		else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Insert");
            pstmt = connection.prepareStatement("INSERT INTO CMP2_SHIP_ (id_, tonnage_, name_) VALUES (?, ?, ?)");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                pstmt.setDouble(2, paccessor.paGetTonnage());
                java.lang.String name_ =
                    paccessor.paGetName();
                if (name_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, name_);
                }
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(1, 4);
                } else {
                    pstmt.setObject(1, id_);
                }
                pstmt.setDouble(2, paccessorGen.paGetDoubleField(pClassMapping.tonnage_NAME));
                java.lang.String name_ =
                    paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (name_==null) {
                    pstmt.setNull(3, 12);
                } else {
                    pstmt.setString(3, name_);
                }
            }
            pstmt.executeUpdate();
		}
		else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "Update");
             pstmt = connection.prepareStatement("UPDATE CMP2_SHIP_ SET tonnage_ = ? , name_ = ?  WHERE id_ = ? ");
            if (paccessor != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(3, 4);
                } else {
                    pstmt.setObject(3, id_);
                }
                pstmt.setDouble(1, paccessor.paGetTonnage());
                java.lang.String name_ =
                    paccessor.paGetName();
                if (name_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, name_);
                }
            }
            else if (paccessorGen != null) {
                java.lang.Integer id_ =
                    pname.encodeOint();
                if (id_==null) {
                    pstmt.setNull(3, 4);
                } else {
                    pstmt.setObject(3, id_);
                }
                pstmt.setDouble(1, paccessorGen.paGetDoubleField(pClassMapping.tonnage_NAME));
                java.lang.String name_ =
                    paccessorGen.paGetStringField(pClassMapping.name_NAME);
                if (name_==null) {
                    pstmt.setNull(2, 12);
                } else {
                    pstmt.setString(2, name_);
                }
            }
            pstmt.executeUpdate();
		}
		} catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} catch(PException pe) {
			throw new PExceptionIO(pe);
		} finally {
		    try {
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (java.sql.SQLException se) {
			    throw new PExceptionIO(se);
			}
		}

		// Changing the current state
		status = nextState;
	}

	public void read(Object conn, PAccessor pa) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		Cmp2_ShipAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof Cmp2_ShipAccessor)
			paccessor = (Cmp2_ShipAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		if (conn==null)
			throw new PExceptionProtocol("Impossible to read without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to read with this connection type.");
        Statement stmt = null;
        ResultSet resultSet = null;
		try {
            stmt = connection.createStatement();
			resultSet = stmt.executeQuery("SELECT tonnage_, name_, id_ FROM CMP2_SHIP_ WHERE id_ = " + pname.encodeOint() + " ");
			if(resultSet.next() == false)
				throw new PExceptionNaming("Invalid PName");

			if (paccessor != null) {
				paccessor.paSetTonnage(
					resultSet.getDouble(1) );
				java.lang.String name = resultSet.getString(2);
				if (resultSet.wasNull()) name = null;
				paccessor.paSetName(
					name );
				java.lang.Integer id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) id = null;
				paccessor.paSetId(
					id );
				java.lang.Integer jonasCMRCabin_Ship_id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) jonasCMRCabin_Ship_id = null;
				if (pClassMapping.jonasCMRCabin_ShipCodingOnly)
					paccessor.paSetJonasCMRCabin_Ship(pClassMapping.jonasCMRCabin_ShipPnc.decodeOint(jonasCMRCabin_Ship_id), conn);
				else
					paccessor.paSetJonasCMRCabin_Ship(pClassMapping.jonasCMRCabin_ShipPnc.decodeOint(jonasCMRCabin_Ship_id).resolve(conn), conn);
				java.lang.Integer jonasCMRCruise_Ship_id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) jonasCMRCruise_Ship_id = null;
				if (pClassMapping.jonasCMRCruise_ShipCodingOnly)
					paccessor.paSetJonasCMRCruise_Ship(pClassMapping.jonasCMRCruise_ShipPnc.decodeOint(jonasCMRCruise_Ship_id), conn);
				else
					paccessor.paSetJonasCMRCruise_Ship(pClassMapping.jonasCMRCruise_ShipPnc.decodeOint(jonasCMRCruise_Ship_id).resolve(conn), conn);
			}
			else if (paccessorGen != null) {
				paccessorGen.paSetDoubleField(pClassMapping.tonnage_NAME,
					resultSet.getDouble(1) );
				java.lang.String name = resultSet.getString(2);
				if (resultSet.wasNull()) name = null;
				paccessorGen.paSetStringField(pClassMapping.name_NAME,
					name );
				java.lang.Integer id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) id = null;
				paccessorGen.paSetOintField(pClassMapping.id_NAME,
					id );
				java.lang.Integer jonasCMRCabin_Ship_id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) jonasCMRCabin_Ship_id = null;
				if (pClassMapping.jonasCMRCabin_ShipCodingOnly)
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCabin_Ship_NAME, pClassMapping.jonasCMRCabin_ShipPnc.decodeOint(jonasCMRCabin_Ship_id), conn);
				else
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCabin_Ship_NAME, pClassMapping.jonasCMRCabin_ShipPnc.decodeOint(jonasCMRCabin_Ship_id).resolve(conn), conn);
				java.lang.Integer jonasCMRCruise_Ship_id = new Integer(resultSet.getInt(3));
				if (resultSet.wasNull()) jonasCMRCruise_Ship_id = null;
				if (pClassMapping.jonasCMRCruise_ShipCodingOnly)
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCruise_Ship_NAME, pClassMapping.jonasCMRCruise_ShipPnc.decodeOint(jonasCMRCruise_Ship_id), conn);
				else
					paccessorGen.paSetRefField(pClassMapping.jonasCMRCruise_Ship_NAME, pClassMapping.jonasCMRCruise_ShipPnc.decodeOint(jonasCMRCruise_Ship_id).resolve(conn), conn);
			}
 		} catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		} catch(PException pe) {
			throw new PExceptionIO(pe);
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null; //garbage the object
			    }
			    if (stmt != null) {
			        stmt.close();
			        stmt = null;
			    }
			} catch (java.sql.SQLException se) {
			    throw new PExceptionIO(se);
			}
		}
		status = nextState;
	}

	public synchronized boolean exist(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized operation");
		status = nextState;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to check the existence without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to check the existence with this connection type.");
        boolean exist = false;
        Statement stmt = null;
		try {
			stmt = connection.createStatement();
			exist = stmt.executeQuery("SELECT * FROM CMP2_SHIP_ WHERE id_ = " + pname.encodeOint() + " ").next();
		}
		catch (java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
        catch (PException e) {
			throw new PExceptionIO(e);
		} finally {
		    try {
			    if (stmt != null) {
			        stmt.close();
			        stmt = null;
			    }
			} catch (java.sql.SQLException se) {
			    throw new PExceptionIO(se);
			}
		}
		return exist;
	}

	public synchronized PName export(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this);
	}

	public synchronized PName export(Object conn, Object hints)
		throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		return pClassMapping.getPBinder().export(conn, this, hints);
	}

	public synchronized void unexport(Object conn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unexport(conn, this.pname);
	}

	public synchronized void bind(PName pn) throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		// The Pbinding must be unbinded only if the PName has been already
		// defined
		if (pname!=null)
			pClassMapping.getPBinder().unbind(this);
		pClassMapping.getPBinder().bind(pn, this);
	}

	public synchronized void unbind() throws PException {
		if (Debug.ON && pClassMapping!=null && pClassMapping.logger.isLoggable(BasicLevel.DEBUG)) pClassMapping.logger.log(BasicLevel.DEBUG, "");
		pClassMapping.getPBinder().unbind(this);
	}

	public PClassMapping getPClassMapping() {
		return pClassMapping;
	}

	public PName getPName() {
		return pname;
	}

	public byte getStatus() {
		return status;
	}

	// IMPLEMENTATION OF METHODS FROM THE PBindingCtrl INTERFACE

	public void setPName(PName pn) {
		pname = pn;
	}

	public void setStatus(byte s) {
		status = s;
	}

 
}
