/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mapping are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2002 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Contact: pascal.dechamboux@rd.francetelecom.com
 *
 * Authors: R. Basset, S. Chassande-Barrioz, P. Dechamboux, N. De Palma,
 *          A. Lefebvre, X. Spengler.
 *
 */
package jorm.rdb.mysql;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.transaction.api.ConnectionWrapper;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.util.HashMap;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;

import java.util.Date;
import java.util.Iterator;

public class Cmp2_CreditCardMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, Loggable {

	protected static final String MAPPER_NAME = "rdb.mysql";
	protected static final String MAPPING_NAME = "jonas";
	//Fields
	protected static final String creditOrganization_NAME = "creditOrganization";
	protected static final String expirationDate_NAME = "expirationDate";
	protected static final String nameOnCard_NAME = "nameOnCard";
	protected static final String number_NAME = "number";
	protected static final String id_NAME = "id";
	protected static final String customer_NAME = "customer";
	//Hidden fields
	protected static final String customer_id_NAME = "customer_id";


	private final static String[][] SUPER_TYPES =  {
    };


	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.Cmp2_CreditCard";

	/** The actual binder which does the binding action */
	private PBinder binder = null;

	/** The meta object of the current jorm class */
	private Class moc = null;

	/** The mapper linked to this PClassMapping */
	private PMapper mapper = null;

	PNamingContext customerPnc = null;
	boolean customerCodingOnly = true;

 
	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	protected Logger logger = null;
	protected LoggerFactory loggerFactory = null;

	public Cmp2_CreditCardMapping() {
		super();
 	}


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMapper m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		PType ptype = mapper.getPTypeSpace().createPType(
			"jorm.Cmp2_CreditCard",
			SUPER_TYPES);
		setPType(ptype);
		String[][] customerICDef = {
		};
		ptype = mapper.getPTypeSpace().createPType(
			"jorm.Cmp2_Customer",
			customerICDef);
		customerPnc.setPType(ptype);
	}

	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNamingContext getPNameManager()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNamingContext getPNameManager(String fn)
		throws UnsupportedOperationException {
		if (customer_NAME.equals(fn))
			return customerPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference");
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
	}

	public boolean isConform(String mapperName) {
		return MAPPER_NAME.equals(mapperName);
	}

	public String getMappingName() {
		return MAPPING_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public void setPNamingContext(PNamingContext nc, boolean co) throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public void setGenClassMapping(PClassMapping pcm) throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	/** This method method must check if the
	  * the fn parameter represents a reference field
	  * Then register the association field name / nc
	  * into the hashMap.
	  * If the jorm class does'nt contains field reference
	  * then an UnsupportedOperationException is throwed */
    public void setPNamingContext(String fn, PNamingContext nc, boolean co)
		throws PException, UnsupportedOperationException {

		// Checking if fn designes a reference field
		if (customer_NAME.equals(fn)) {
			// checking if the PNamingContext is able to encode
 			if (!nc.codingSupported(PNamingContext.CTOINT)) {
				throw new PExceptionTyping("The " + fn
					+ " field doesn't support the Int encoding");
			}
			customerPnc = nc;
			customerCodingOnly = co;
			return;
		}
		throw new UnsupportedOperationException(
			"No field reference with the name " + fn + " in this class");
	}

	public void setGenClassMapping(String fn, PClassMapping pcm) throws UnsupportedOperationException {
		throw new UnsupportedOperationException("No field reference to a generic class with the name " + fn + " in this class");
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new Cmp2_CreditCardBinding();
		try {
			binding.init(this);
		}
		catch(PExceptionProtocol pep) {
		}
		return binding;
	}
	public void initMappingStructures(Object conn, byte cleanup)
		throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");

		Connection connection = null;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to initialize the mapping structures without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to initialize the mapping structures with this connection type.");
		try {
			//Ask to the database if the table already exists
			// Be carefull some driver does not support the table name partten,
			// then the table name is searched in the resultSet.
			ResultSet rs = connection.getMetaData().getTables(null,null, "CMP2_CREDITCARD_" ,null);
			boolean existTable = false;
			while(rs.next() && !existTable)
				existTable = "CMP2_CREDITCARD_".equalsIgnoreCase(rs.getString(3));
			rs.close();
			if (existTable) {
				if (cleanup==CLEANUP_REMOVEDATA)
					removeMappingStructures(conn, true);
				else if (cleanup==CLEANUP_REMOVEALL) {
					removeMappingStructures(conn, false);
					existTable = false;
				}
			}
			if (!existTable) {
				Statement stmt = connection.createStatement();
				stmt.execute("CREATE TABLE CMP2_CREDITCARD_ (id_ INTEGER NOT NULL, creditOrganization_ VARCHAR(254), expirationDate_ DATE, nameOnCard_ VARCHAR(254), number_ VARCHAR(254), customer_id INTEGER, PRIMARY KEY (id_))");
				stmt.close();
			}
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
	}
	public void removeMappingStructures(Object conn, boolean contentonly)
		throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		Connection connection = null;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to initialize the mapping structures without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to initialize the mapping structures with this connection type.");
		try {
			Statement stmt = connection.createStatement();
			if (contentonly)
				// Remove all data only
				stmt.execute("DELETE FROM CMP2_CREDITCARD_");
			else
				// Remove all data and the structures
				stmt.execute("DROP TABLE CMP2_CREDITCARD_");
			stmt.close();
		}
		catch(java.sql.SQLException se) {
			throw new PExceptionIO(se);
		}
	}

    public Iterator getPNameIterator(Object conn) throws PException {
                Connection connection = null;
		if (conn==null)
			throw new PExceptionProtocol("Impossible to initialize the mapping structures without connection");
		else if (conn instanceof Connection)
			connection = (Connection) conn;
		else if (conn instanceof ConnectionWrapper)
			connection = (Connection) ((ConnectionWrapper) conn).getNonCCIConnection();
		else
			throw new PExceptionProtocol("Impossible to initialize the mapping structures with this connection type.");
		try {
			PreparedStatement ps = connection.prepareStatement(
                "SELECT id_ FROM CMP2_CREDITCARD_");
            return new Cmp2_CreditCardResultIterator(ps, ps.executeQuery(), binder);
        } catch (java.sql.SQLException e) {
            throw new PExceptionIO(e);
        }
    }

    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
		setPNamingContext(
            customer_NAME,
            rc.getPNameManager(JORM_CLASS_NAME, customer_NAME, "jorm.Cmp2_Customer"),
            rc.isCodingOnly(JORM_CLASS_NAME, customer_NAME));
    }

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}


}
class Cmp2_CreditCardResultIterator implements Iterator
        {
        ResultSet rs;
		PBinder pnc;
        boolean hasNext;
        boolean isForw;
		PreparedStatement ps;
        Cmp2_CreditCardResultIterator(PreparedStatement ps, ResultSet r,  PBinder pnc)
			throws java.sql.SQLException {
			this.ps = ps;
			this.pnc = pnc;
            rs = r;
            hasNext = rs.next();
            isForw = true;
        }

		public void finalize() throws Throwable {
			if (rs != null)
				rs.close();
			if (ps != null)
				ps.close();
			super.finalize();
		}

        public boolean hasNext() {
            if (isForw) {
                return hasNext;
            }
            else {
                isForw = true;
                try {
                    return hasNext = rs.next();
                } catch(Exception e) {
                    e.printStackTrace();
                    return false;
                }
            }
        }

        public Object next() throws NoSuchElementException {
            try {
                if (!isForw) {
                    hasNext = rs.next();
                    isForw = true;
                }
                if (!hasNext) {
                    throw new NoSuchElementException();
                }
                isForw = false;
                return pnc.decodeOint( new Integer(rs.getInt(1)));
            } catch (Exception e) {
                throw new NoSuchElementException(
                    "Impossible to decode to a PName: " + e.getMessage());
            }
        }

        public void remove()
            throws UnsupportedOperationException, IllegalStateException {
            throw new UnsupportedOperationException("");
        }

}
